/*************************************************************************
 *
 *   Used with ICCARM and AARM.
 *
 *    (c) Copyright IAR Systems 2008
 *
 *    File name   : dataFlashLoader.c
 *    Description : Flash loader for Embedded Artits LPC313x board. 
 *                  Atmel AT45 SPINOR Flash.
 *                  This flash loader is intended for programming an application
 *                  which will be loaded by second level boot loader into the 
 *                  external RAM. The Block 0 and Block 1 of the SPI NOR flash
 *                  memory are reserved for the second level boot loader. 
 *
 *    History :
 *    1. Date        : 26.3.2009
 *       Author      : Stoyan Choynev
 *       Description : Initial Revision
 *
 *    $Revision: #3 $
 **************************************************************************/

/** include files **/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <intrinsics.h>
#include <NXP\iolpc3130.h>

#include "flash_loader.h"
#include "flash_loader_extra.h"
#include "flash_config.h"

#include "drv_cgu.h"
#include "drv_spi.h"
#include "drv_spinor.h"

/** local definitions **/
// Second level bootloader data
#pragma pack(1)

typedef union _Sec_level_boot_data_t
{
  unsigned char Data[5];
  struct
  {
    unsigned long DestAddr;
    unsigned char Type;
  };
} Sec_level_boot_data_t;

#pragma pack()

typedef enum _Sec_level_boot_page_type_t
{
  REGULAR_PAGE = 0, ENCRYPTED_PAGE,
  INVALID_PAGE = 0xFF
} Sec_level_boot_page_type_t;

#define SIZE_BOOTL2_DATA                sizeof(Sec_level_boot_data_t)
#define PAGES_PER_BLOCK                 8

#define APP_CHIP_SEL_RANGE              0x4000000
/*Block 0 And Block 1 are reserved for bootloader level 2*/
#define BOOTLOADER_L2_IMAGE_MAX_SIZE   2*PAGES_PER_BLOCK  //16 Pages (the actual size depends of page size)

/** default settings **/

/** external functions **/

/** external data **/

/** internal functions **/
#if USE_ARGC_ARGV
const char* FlFindOption(char* option, int with_value, int argc, char const* argv[]);
#endif /*USE_ARGC_ARGV*/
/** public data **/

/** private data **/
static unsigned int pageSize;
static unsigned int pageNum;
// current page of block
static unsigned int memoryPage;

// "binary" page size
static unsigned int RepPageSize;

// Current table page address
static unsigned int TablePage;

// offset in table page
static unsigned int TablePageOffset;

/** public functions **/
/*************************************************************************
 * Function Name: Dly_us
 * Parameters: Delay in us
 *
 * Return:None
 *
 * Description: None
 *
 *************************************************************************/

void Dly_us(Int32U Dly)
{
  Dly *= 12;
  Timer0Ctrl = 0;               // stop counting
  Timer0Load =  Dly;            // load period
  Timer0Clear = 0;              // clear timer pending interrupt
  Timer0Ctrl_bit.Enable = 1;    // enable counting
  while(Timer0Value <= Dly);
  Timer0Ctrl_bit.Enable = 0;     // stop counting
}


/*************************************************************************
 * Function Name: FlashInit
 * Parameters: Flash Base Address, image_size
 *
 * Return:  RESULT_ERROR - Init Failed
 *          OVERRIDE_LAYOUT | OVERRIDE_PAGESIZE - Init Successful
 *                                                Override layout and
 *                                                pagesize 
 * Description: Init flash and build layout.
 *
 *************************************************************************/
#if USE_ARGC_ARGV
 uint32_t FlashInit(void *base_of_flash, uint32_t image_size,
                    int argc, char const *argv[])
#else
 uint32_t FlashInit(void *base_of_flash, uint32_t image_size)
#endif /*USE_ARGC_ARGV*/
{
    unsigned int cs_range;
    unsigned int remainder;

    // Temporary buffer used string convert.
    char pString[50];
    
    InitClock();

    if(InitSPINOR()) return RESULT_ERROR;

    RepPageSize = pageSize = SPINOR_GetPageSize();
    pageNum =  SPINOR_GetPageNum();

    cs_range = APP_CHIP_SEL_RANGE;
    memoryPage = BOOTLOADER_L2_IMAGE_MAX_SIZE;
      
    SPINOR_Erase_Block(memoryPage);

    RepPageSize     &= ~0xFFUL;
      
    if (0 == (0xFF & pageSize))
    {// "binary" page size layout
      TablePageOffset  = 0;
      TablePage = memoryPage;
      memoryPage++;
    }
    // Override blocksize
    sprintf(pString, "%u 0x%x, ", cs_range/(pageSize<<3), (RepPageSize<<3));

    strcpy((char*) theFlashParams.buffer, pString);

    remainder = cs_range%(pageSize<<3);
    
    if(remainder)
    {
      // Rest of CS range
      sprintf(pString,"1 0x%x",remainder);
      strcat(theFlashParams.buffer,pString);
    }
    // override page size
    theFlashParams.offset_into_block = RepPageSize;
    
    return OVERRIDE_LAYOUT | OVERRIDE_PAGESIZE;
}
/*************************************************************************
 * Function Name: FlashErase
 * Parameters:void *block_start,
 *                uint32_t block_size
 *
 * Return: RESULT_OK - Erase Successful
 *         
 * Description: None
 *
 *************************************************************************/

uint32_t FlashErase(void *block_start,
                    uint32_t block_size)
{
  return RESULT_OK;
}

/*************************************************************************
 * Function Name: FlashWrite
 * Parameters: void *block_start         
 *            uint32_t offset_into_block 
 *            uint32_t count,            
 *            char const *buffer         
 *
 * Return:  RESULT_ERROR - Write Failed
 *          RESULT_OK    - Write Successful      
 *
 * Description: Write Data
 *
 *************************************************************************/
uint32_t FlashWrite(void *block_start,
                    uint32_t offset_into_block,
                    uint32_t count,
                    char const *buffer)
{
unsigned int Size = 0;
    
  Sec_level_boot_data_t Bootl2Data;
     
  Bootl2Data.Type = REGULAR_PAGE;
     
  while(Size < count)
  {
    Bootl2Data.DestAddr = (unsigned long)block_start+offset_into_block+Size;
      
    if(memoryPage >= pageNum)
    {
      return RESULT_ERROR;
    }      
        
    SPINOR_Write_Page(memoryPage,(unsigned char *)(buffer+Size));
        
    if(0xFF & pageSize)
    {
      SPINOR_Write_Bytes(memoryPage,pageSize-5,&Bootl2Data.Data[0],SIZE_BOOTL2_DATA);
    }
    else
    {
          // write L2 bootloader data in table page
      SPINOR_Write_Bytes(TablePage,TablePageOffset,&Bootl2Data.Data[0],SIZE_BOOTL2_DATA);          
      TablePageOffset += SIZE_BOOTL2_DATA;
          
      if ((pageSize - SIZE_BOOTL2_DATA) < TablePageOffset)
      {
        TablePageOffset  = 0;
        TablePage = ++memoryPage;
            
        if(0 == (TablePage%PAGES_PER_BLOCK)) SPINOR_Erase_Block(TablePage);         
      }
    }
        
        
    Size += RepPageSize;
    memoryPage++;
           
    if(0 == (memoryPage%PAGES_PER_BLOCK)) SPINOR_Erase_Block(memoryPage);
        
  }

  return RESULT_OK;
}


/** private functions **/
#if USE_ARGC_ARGV
const char* FlFindOption(char* option, int with_value, int argc, char const* argv[])
{
  int i;

  for (i = 0; i < argc; i++)
  {
    if (strcmp(option, argv[i]) == 0)
    {
      if (with_value)
      {
        if (i + 1 < argc)
          return argv[i + 1]; // The next argument is the value.
        else
          return 0; // The option was found but there is no value to return.
      }
      else
      {
        return argv[i]; // Return the flag argument itself just to get a non-zero pointer.
      }
    }
  }
  return 0;
}
#endif /*USE_ARGC_ARGV*/
